"""
"""

from globalpayments.api.entities.enums import EntryMethod


class PaymentMethod(object):
    payment_method_type = None


class CardData(object):
    card_present = False
    cvn = None
    cvn_presence_indicator = None
    number = None
    exp_month = None
    exp_year = None
    reader_present = False


class TrackData(object):
    value = None
    entry_method = EntryMethod.Swipe


class Authable(object):
    def authorize(self, amount=None):
        pass


class Chargable(object):
    def charge(self, amount=None):
        pass


class Balanceable(object):
    def balance_inquiry(self, inquiry_type=None):
        pass


class Editable(object):
    def edit(self, amount=None):
        pass


class Encryptable(object):
    encryption_data = None


class PinProtected(object):
    pin_block = None


class PrePayable(object):
    def add_value(self, amount=None):
        pass


class Refundable(object):
    def refund(self, amount=None):
        pass


class Reversable(object):
    def reverse(self, amount=None):
        pass


class Tokenizable(object):
    token = None

    def tokenize(self):
        pass


class Verifiable(object):
    def verify(self):
        pass


"""
"""

from globalpayments.api.entities.enums import (
    FraudFilterMode,
    HppVersion,
    ReservationProviders,
)
from globalpayments.api.entities.exceptions import ConfigurationException
from globalpayments.api.gateways import (
    PayPlanConnector, PorticoConnector, RealexConnector, TableServiceConnector)


class HostedPaymentConfig(object):
    """
    Hosted Payment Page (HPP) configuration
    This configuration is used when constructing HPP requests to be used by
    a client library (JS, iOS, Android).
    """

    #  Allow card to be stored within the HPP
    card_storage_enabled = None
    #  Allow Dynamic Currency Conversion (DCC) to be available
    dynamic_currency_conversion_enabled = None
    #  Allow a consumer's previously stored cards to be shown
    display_saved_cards = None
    #  Manner in which the fraud filter should operate
    fraud_filter_mode = FraudFilterMode.NONE
    #  The desired language for the HPP
    language = None
    #  Text for the HPP's submit button
    payment_button_text = None
    #  URL to receive `POST` data of the HPP's result
    response_url = None
    #  Denotes if Transaction Stability Score (TSS) should be active
    request_transaction_stability_score = None
    #  Specify HPP version
    version = HppVersion.VERSION_1
    #  iFrame Optimisation - dimensions
    post_dimensions = None
    #  iFrame Optimisation - response
    post_response = None


class ServicesConfig(object):
    """
    Configuration for connecting to a payment gateway
    """

    #  Account's site ID
    site_id = None
    #  Account's license ID
    license_id = None
    #  Account's device ID
    device_id = None
    #  Account's username
    username = None
    #  Account's password
    password = None
    """
    Integration's developer ID
    This is provided at the start of an integration's certification
    """
    developer_id = None
    """
    Integration's version number
    This is provided at the start of an integration's certification
    """
    version_number = None
    #  Account's secret API Key
    secret_api_key = None
    #  Account's account ID
    account_id = None
    # Account's merchant ID
    merchant_id = None
    #  Account's rebate password
    rebate_password = None
    #  Account's refund password
    refund_password = None
    #  Account's shared secret
    shared_secret = None
    #  Channel for an integration's transactions (e.g. "internet")
    channel = None
    #  Hosted Payment Page (HPP) configuration
    hosted_payment_config = None
    #  Connection details for physical card reader device
    device_connection_config = None
    #  Connection details for the reservation service
    reservation_provider = None
    #  Gateway Service URL
    service_url = None
    #  Timeout value for gateway communication (in milliseconds)
    timeout = None

    def __init__(self):
        self.timeout = 65000

    def validate(self):
        #  portico api key
        if self.secret_api_key is not None:
            if (self.site_id is not None or self.license_id is not None
                    or self.device_id is not None or self.username is not None
                    or self.password is not None):
                raise ConfigurationException(
                    """Configuration contains both secret api key and legacy credentials.
                    These are mutually exclusive.""")

        #  legacy portico
        if (self.site_id is not None or self.license_id is not None
                or self.device_id is not None or self.username is not None
                or self.password is not None):
            if (self.site_id is None or self.license_id is None
                    or self.device_id is None or self.username is None
                    or self.password is None):
                raise ConfigurationException(
                    """Site, License, Device, Username and Password should all have a
                    values for this configuration.""")

        #  realex
        if self.merchant_id is not None or self.shared_secret is not None:
            if self.merchant_id is None:
                raise ConfigurationException(
                    'merchant_id is required for this configuration.')
            if self.shared_secret is None:
                raise ConfigurationException(
                    'shared_secret is required for this configuration.')

        #  service url
        if self.service_url is None:
            pass


class ConfiguredServices(object):
    gateway_connector = None
    recurring_connector = None
    device_interface = None
    device_controller = None
    reservation_connector = None


SERVICE_CONTAINER_INSTANCE = None


class ServicesContainer(object):
    """
    Maintains references to the currently configured gateway/device objects
    The 'ServicesContainer.configure' method is the only call
    required of the integrator to configure the SDK's various gateway/device
    interactions. The configured gateway/device objects are handled
    internally by exposed APIs throughout the SDK.
    """

    _configurations = None

    @staticmethod
    def instance():
        global SERVICE_CONTAINER_INSTANCE

        if SERVICE_CONTAINER_INSTANCE is not None:
            return SERVICE_CONTAINER_INSTANCE
        raise ConfigurationException('Services container not configured.')

    @staticmethod
    def configure(config, config_name="default"):
        global SERVICE_CONTAINER_INSTANCE

        if not isinstance(config, ServicesConfig):
            raise ConfigurationException(
                'config must be of type ServiceConfig')

        config.validate()

        cs = ConfiguredServices()

        #  configure devices
        # if config.device_connection_config is not None:
        #     if config.device_connection_config.device_type is DeviceType.PAX_S300:
        #         device_controller = PaxController(config.device_connection_config)
        #     elif config.device_connection_config.device_type is DeviceType.HSIP_ISC250:
        #         device_controller = HeartSipController(config.device_connection_config)
        #
        #     if device_controller is not None:
        #         device_interface = device_controller.configure_interface()

        # configure reservations
        if config.reservation_provider is not None:
            if config.reservation_provider is ReservationProviders.FreshTxt:
                cs.reservation_connector = TableServiceConnector()
                cs.reservation_connector.service_url = 'https://www.freshtxt.com/api31/'
                cs.reservation_connector.timeout = config.timeout

        # configure gateways
        if config.merchant_id is not None:
            cs.gateway_connector = RealexConnector()
            cs.gateway_connector.account_id = config.account_id
            cs.gateway_connector.channel = config.channel
            cs.gateway_connector.merchant_id = config.merchant_id
            cs.gateway_connector.rebate_password = config.rebate_password
            cs.gateway_connector.refund_password = config.refund_password
            cs.gateway_connector.shared_secret = config.shared_secret
            cs.gateway_connector.timeout = config.timeout
            cs.gateway_connector.service_url = config.service_url
            cs.gateway_connector.hosted_payment_config = config.hosted_payment_config
            cs.recurring_connector = cs.gateway_connector
        else:
            cs.gateway_connector = PorticoConnector()
            cs.gateway_connector.site_id = config.site_id
            cs.gateway_connector.license_id = config.license_id
            cs.gateway_connector.device_id = config.device_id
            cs.gateway_connector.username = config.username
            cs.gateway_connector.password = config.password
            cs.gateway_connector.secret_api_key = config.secret_api_key
            cs.gateway_connector.developer_id = config.developer_id
            cs.gateway_connector.version_number = config.version_number
            cs.gateway_connector.timeout = config.timeout
            cs.gateway_connector.service_url = config.service_url \
                + '/Hps.Exchange.PosGateway/PosGatewayService.asmx'

            cs.recurring_connector = PayPlanConnector()
            cs.recurring_connector.secret_api_key = config.secret_api_key
            cs.recurring_connector.timeout = config.timeout
            cs.recurring_connector.service_url = config.service_url + '/Portico.PayPlan.v2/'

        if SERVICE_CONTAINER_INSTANCE is None:
            SERVICE_CONTAINER_INSTANCE = ServicesContainer()

        SERVICE_CONTAINER_INSTANCE.add_configuration(config_name, cs)

    def __init__(self):
        self._configurations = {}

    def add_configuration(self, config_name, config):
        self._configurations[config_name] = config

    def get_client(self, config_name):
        if config_name in self._configurations:
            return self._configurations[config_name].gateway_connector
        return None

    def get_device_interface(self, config_name):
        if config_name in self._configurations:
            return self._configurations[config_name].device_interface
        return None

    def get_device_controller(self, config_name):
        if config_name in self._configurations:
            return self._configurations[config_name].device_controller
        return None

    def get_recurring_client(self, config_name):
        if config_name in self._configurations:
            return self._configurations[config_name].recurring_connector
        return None

    def get_reservation_service(self, config_name):
        if config_name in self._configurations:
            return self._configurations[config_name].reservation_connector
        return None



class ValidationClause(object):
    parent = None
    target = None
    property_name = None
    callback = None
    message = None
    precondition = None

    def __init__(self, parent, target, property_name, precondition=False):
        self.parent = parent
        self.target = target
        self.property_name = property_name
        self.precondition = precondition

    def is_not_none(self, message=None):
        self.callback = (
            lambda builder: self._get_property_value(builder, self.property_name) is not None
        )
        self.message = message if message is not None else 'property `{}` is None'.format(
            self.property_name)

        if self.precondition:
            return self.target

        return self.parent.of(self.target.type_name) \
            .with_constraint(self.target.constraint_name, self.target.constraint_value)

    def is_none(self, message=None):
        self.callback = (
            lambda builder: self._get_property_value(builder, self.property_name) is None
        )
        self.message = message if message is not None else 'property `{}` is not None'.format(
            self.property_name)

        if self.precondition:
            return self.target

        return self.parent.of(self.target.type_name) \
            .with_constraint(self.target.constraint_name, self.target.constraint_value)

    def equals(self, expected, message=None):
        self.callback = (
            lambda builder: self._get_property_value(builder, self.property_name) is expected
        )
        self.message = message if message is not None else 'property `{}` does not equal `{}`'.format(
            self.property_name, str(expected))

        if self.precondition:
            return self.target

        return self.parent.of(self.target.type_name) \
            .with_constraint(self.target.constraint_name, self.target.constraint_value)

    def does_not_equal(self, expected, message=None):
        self.callback = (
            lambda builder: self._get_property_value(builder, self.property_name) is not expected
        )
        self.message = message if message is not None else 'property `{}` is equals `{}`'.format(
            self.property_name, str(expected))

        if self.precondition:
            return self.target

        return self.parent.of(self.target.type_name) \
            .with_constraint(self.target.constraint_name, self.target.constraint_value)

    @staticmethod
    def _get_property_value(obj, comp):
        if obj is None:
            return None

        try:
            return getattr(obj, comp)
        except AttributeError as _exc:
            return None



class DebitMac(object):
    transaction_code = None
    transmission_number = None
    bank_response_code = None
    mac_key = None
    pin_key = None
    field_key = None
    trace_number = None
    message_authentication_code = None



from globalpayments.api.services.batch_service import BatchService
from globalpayments.api.services.recurring_service import RecurringService
from globalpayments.api.services.reporting_service import ReportingService



import globalpayments as gp
from globalpayments.api.entities.enums import TransactionType


class BatchService(object):
    @staticmethod
    def close_batch(config_name='default'):
        _response = gp.api.builders.ManagementBuilder(
            TransactionType.BatchClose).execute(config_name)
        return gp.api.entities.BatchSummary()



from globalpayments.api.builders.validations.validation_clause import ValidationClause
from globalpayments.api.builders.validations.validations import Validations
from globalpayments.api.builders.validations.validation_target import ValidationTarget

__all__ = ['Validations', 'ValidationTarget', 'ValidationClause']



from globalpayments.api.payment_methods.credit import Credit, CreditCardData, CreditTrackData
from globalpayments.api.payment_methods.debit import DebitTrackData
from globalpayments.api.payment_methods.ebt import EBTCardData, EBTTrackData
from globalpayments.api.payment_methods.echeck import ECheck
from globalpayments.api.payment_methods.giftcard import GiftCard
from globalpayments.api.payment_methods.payment_interfaces import *
from globalpayments.api.payment_methods.transaction_reference import TransactionReference

__all__ = ['EBTCardData', 'TransactionReference']



class ApiException(Exception):
    """
    a general error occurred
    """

    def __init__(self, message, inner_exception=None):
        Exception.__init__(self, message)
        self.message = message
        self.inner_exception = inner_exception


class BuilderException(ApiException):
    """
    A builder error occurred. Check the method calls against the builder.
    """

    def __init__(self, message=None):
        ApiException.__init__(self, message)


class ConfigurationException(ApiException):
    """
    An account or SDK configuration error occurred.
    """

    def __init__(self, message):
        ApiException.__init__(self, message)


class GatewayException(ApiException):
    """
    An error occurred on the gateway.
    """

    #  the gateway response code
    response_code = None
    #  the gateway response message
    response_message = None

    def __init__(self,
                 message,
                 response_code=None,
                 response_message=None,
                 inner_exception=None):
        ApiException.__init__(self, message, inner_exception)
        self.response_code = response_code
        self.response_message = response_message


class MessageException(ApiException):
    """
    A message to/from the device caused an error.
    """

    def __init__(self, message, inner_exception=None):
        ApiException.__init__(self, message, inner_exception)


class UnsupportedTransactionException(ApiException):
    """
    The built transaction is not supported for the gateway or payment method.
    """

    def __init__(self, message=None):
        ApiException.__init__(
            self, ('Transaction type not supported for this payment method.'
                   if message is None else message))



from enum import Enum, IntEnum


class AliasAction(Enum):
    Create = 'CREATE'
    Add = 'ADD'
    Delete = 'DELETE'


class AddressType(Enum):
    """
    Indicates an address type.
    """

    #  Indicates a billing address.
    Billing = 0
    #  Indicates a shipping address.
    Shipping = 1


class DeviceType(Enum):
    """
    Indicates a device type for out of scope / semi-integrated devices.
    """

    #  Indicates a Pax S300 device.
    PAX_S300 = 0
    #  Indicates a HeartSIP iSC250 device.
    HSIP_ISC250 = 1


class ECommerceChannel(Enum):
    """
    Identifies eCommerce vs mail order / telephone order (MOTO) transactions.
    """
    #  Identifies eCommerce transactions.
    ECOM = 'ECOM'
    #  Identifies mail order / telephone order (MOTO) transactions.
    MOTO = 'MOTO'


class EmvChipCondition(Enum):
    """
    Indicates the chip condition for failed EMV chip reads
    """
    """
    Use this condition type when the current chip read failed but
    the previous transaction on the same device was either a
    successful chip read or was not a chip transaction.
    """
    ChipFailedPreviousSuccess = 0
    """
    Use this condition type when the current chip read failed and
    the previous transaction on the same device was also an
    unsuccessful chip read.
    """
    ChipFailedPreviousFailed = 1


class InquiryType(Enum):
    """
    Indicates an inquiry type.
    """

    Standard = "STANDARD"
    #  Indicates a foodstamp inquiry.
    FoodStamp = "FOODSTAMP"
    #  Indicates a cash inquiry.
    Cash = "CASH"
    Points = "POINTS"


class PaymentMethodType(IntEnum):
    """
    Indicates a payment method type.
    """
    """
    Indicates a payment method reference.
    Should be accompanied by a gateway transaction ID.
    """
    Reference = 0
    """
    Indicates a credit or PIN-less debit account.
    Should be accompanied by a token, card number, or track data.
    """
    Credit = 1 << 1
    """
    Indicates a PIN debit account.
    Should be accompanied by track data and a PIN block.
    """
    Debit = 1 << 2
    """
    Indicates an EBT account.
    Should be accompanied by track data and a PIN block.
    """
    EBT = 1 << 3
    """
    Indicates cash as the payment method.
    """
    Cash = 1 << 4
    """
    Indicates an ACH/eCheck account.
    Should be accompanied by a token or an account number and routing number.
    """
    ACH = 1 << 5
    """
    Indicates a gift/loyalty/stored value account.
    Should be accompanied by a token, card number, alias, or track data.
    """
    Gift = 1 << 6
    """
    Indicates a recurring payment method.
    Should be accompanied by a payment method key.
    """
    Recurring = 1 << 7


class EntryMethod(Enum):
    """
    Indicates how the payment method data was obtained.
    """

    #  Indicates manual entry.
    Manual = 'manual'
    #  Indicates swipe entry.
    Swipe = 'swipe'
    #  Indicates proximity/contactless entry.
    Proximity = 'proximity'


class GiftEntryMethod(Enum):
    """
    Indicates how the gift/loyalty/stored value account data was obtained.
    """

    #  Indicates swipe entry.
    Swipe = 0
    #  Indicates an alias was entered.
    Alias = 1
    #  Indicates manual entry.
    Manual = 2


class TransactionType(IntEnum):
    """
    Indicates the transaction type.
    """

    #  Indicates a decline.
    Decline = 0
    #  Indicates an account verify.
    Verify = 1 << 0
    #  Indicates a capture/add to batch
    Capture = 1 << 1
    #  Indicates an authorization without capture
    Auth = 1 << 2
    #  Indicates a refund
    Refund = 1 << 3
    #  Indicates a reversal
    Reversal = 1 << 4
    #  Indicates a sale/charge/auth with capture
    Sale = 1 << 5
    #  Indicates an edit
    Edit = 1 << 6
    #  Indicates a void
    Void = 1 << 7
    #  Indicates value should be added
    AddValue = 1 << 8
    #  Indicates a balance inquiry
    Balance = 1 << 9
    #  Indicates an activation
    Activate = 1 << 10
    #  Indicates an alias should be added
    Alias = 1 << 11
    #  Indicates the payment method should be replaced
    Replace = 1 << 12
    #  Indicates a reward
    Reward = 1 << 13
    #  Indicates a deactivation
    Deactivate = 1 << 14
    #  Indicates a batch close
    BatchClose = 1 << 15
    #  Indicates a resource should be created
    Create = 1 << 16
    #  Indicates a resource should be deleted
    Delete = 1 << 17
    #  Indicates a benefit withdrawal
    BenefitWithdrawal = 1 < 18
    #  Indicates a resource should be fetched
    Fetch = 1 << 19
    #  Indicates a resource type should be searched
    Search = 1 << 20
    #  Indicates a hold
    Hold = 1 << 21
    #  Indicates a release
    Release = 1 << 22
    #  Indicates a 3DSecure signature verification
    VerifySignature = 1 << 23
    #  Indicates a 3DSecure enrollment verification
    VerifyEnrolled = 1 << 24


class TransactionModifier(IntEnum):
    """
    Indicates if a transaction should be specialized.
    """

    #  Indicates no specialization
    NoModifier = 0
    #  Indicates an incremental transaction
    Incremental = 1 << 1
    #  Indicates an additional transaction
    Additional = 1 << 2
    #  Indicates an offline transaction
    Offline = 1 << 3
    #  Indicates a commercial request transaction
    LevelII = 1 << 4
    #  Indicates a fraud decline transaction
    FraudDecline = 1 << 5
    #  Indicates a chip decline transaction
    ChipDecline = 1 << 6
    #  Indicates a cash back transaction
    CashBack = 1 << 7
    #  Indicates a voucher transaction
    Voucher = 1 << 8
    #  Indicates a secure 3d transaction
    Secure3D = 1 << 9
    #  Indicates a hosted payment transaction
    HostedRequest = 1 << 10
    #  Indicates a recurring transaction
    Recurring = 1 << 11


class CvnPresenceIndicator(Enum):
    """
    Indicates CVN presence at time of payment.
    """

    #  Indicates
    Present = 1
    #  Indicates CVN was present but illegible
    Illegible = 2
    #  Indicates CVN was not present
    NotOnCard = 3
    #  Indicates was not requested
    NotRequested = 4


class TaxType(Enum):
    """
    Indicates the tax type.
    """

    #  Indicates tax was not used
    NotUsed = 'NOTUSED'
    #  Indicates sales tax was applied
    SalesTax = 'SALESTAX'
    #  Indicates tax exemption
    TaxExempt = 'TAXEXEMPT'


class CurrencyType(Enum):
    """
    Indicates the currency type.
    """

    #  Indicates true currency
    Currency = 'CURRENCY'
    #  Indicates loyalty points
    Points = 'POINTS'
    #  Indicates cash benefits
    CashBenefits = 'CASH_BENEFITS'
    #  Indicates food stamps
    FoodStamps = 'FOODSTAMPS'
    #  Indicates vouchers
    Voucher = 'VOUCHER'


class AccountType(Enum):
    """
    Indicates the account type for ACH/eCheck transactions.
    """

    #  Indicates a checking account
    Checking = 'CHECKING'
    #  Indicates a savings account
    Savings = 'SAVINGS'


class CheckType(Enum):
    """
    Indicates the check type for ACH/eCheck transactions.
    """

    #  indicates a personal check
    Personal = 'PERSONAL'
    #  Indicates a business check
    Business = 'BUSINESS'
    #  Indicates a payroll check
    Payroll = 'PAYROLL'


class SecCode(Enum):
    """
    Indicates the NACHA standard entry class (SEC) code for ACH/eCheck transactions.
    """

    #  Indicates prearranged payment and deposit (PPD)
    PPD = 'PPD'
    #  Indicates cash concentration or disbursement (CCD)
    CCD = 'CCD'
    #  Indicates point of purchase entry (POP)
    POP = 'POP'
    #  Indicates internet initiated entry (WEB)
    WEB = 'WEB'
    #  Indicates telephone initiated entry (TEL)
    TEL = 'TEL'
    #  Indicates verification only
    EBRONZE = 'EBRONZE'


class ReportType(IntEnum):
    """
    Indicates the report type.
    """

    #  Indicates a FindTransactions report
    FindTransactions = 0
    #  Indicates an Activity report
    Activity = 1 << 1
    #  Indicates a BatchDetail report
    BatchDetail = 1 << 2
    #  Indicates a BatchHistory report
    BatchHistory = 1 << 3
    #  Indicates a BatchSummary report
    BatchSummary = 1 << 4
    #  Indicates an OpenAuth report
    OpenAuths = 1 << 5
    #  Indicates a Search report
    Search = 1 << 6
    #  Indicates a TransactionDetail report
    TransactionDetail = 1 << 7


class TimeZoneConversion(Enum):
    """
    Indicates how timezones should be handled.
    """

    #  Indicates time is in coordinated universal time (UTC).
    UTC = 'UTC'
    #  Indicates the merchant is responsible for timezone conversions.
    Merchant = 'Merchant'
    #  Indicates the datacenter, gateway, or processor is responsible for timezone conversions.
    Datacenter = 'Datacenter'


class RecurringType(Enum):
    """
    Indicates the type of recurring schedule.
    """

    #  Indicates a fixed number of payments
    Fixed = 'Fixed'
    #  Indicates a variable number of payments.
    Variable = 'Variable'


class RecurringSequence(Enum):
    """
    Indicates when a transaction is ran in a recurring schedule.
    """

    #  Indicates the transaction is the first of a recurring schedule.
    First = 'First'
    #  Indicates the transaction is a subsequent payment of a recurring schedule.
    Subsequent = 'Subsequent'
    #  Indicates the transaction is the last of a recurring schedule.
    Last = 'Last'


class EmailReceipt(Enum):
    """
    Indicates when an email receipt should be sent for the transaction.
    Currently only used in recurring schedules.
    """

    #  Indicates an email receipt should never be sent.
    Never = 'Never'
    #  Indicates an email receipt should always be sent.
    All = 'All'
    #  Indicates an email receipt should only be sent on approvals.
    Approvals = 'Approvals'
    #  Indicates an email receipt should only be sent on declines.
    Declines = 'Declines'


class PaymentSchedule(Enum):
    """
    Indicates when in the month a recurring schedule should run.
    """

    #  Indicates a specified date.
    Dynamic = 'Dynamic'
    #  Indicates the first of the month.
    FirstDayOfTheMonth = 'FirstDayOfTheMonth'
    #  Indicates the last of the month.
    LastDayOfTheMonth = 'LastDayOfTheMonth'


class ScheduleFrequency(Enum):
    """
    Indicates the frequency of a recurring schedule.
    """

    #  Indicates a schedule should process payments weekly.
    Weekly = 'Weekly'
    #  Indicates a schedule should process payments bi-weekly (every other week).
    BiWeekly = 'Bi-Weekly'
    #  Indicates a schedule should process payments bi-monthly (twice a month).
    BiMonthly = 'Bi-Monthly'
    #  Indicates a schedule should process payments semi-monthly (every other month).
    SemiMonthly = 'Semi-Monthly'
    #  Indicates a schedule should process payments monthly.
    Monthly = 'Monthly'
    #  Indicates a schedule should process payments quarterly.
    Quarterly = 'Quarterly'
    #  Indicates a schedule should process payments semi-annually
    SemiAnnually = 'Semi-Annually'
    #  Indicates a schedule should process payments annually. (twice a year).
    Annually = 'Annually'


class ReasonCode(Enum):
    """
    Indicates a reason for the transaction.
    This is typically used for returns/reversals.
    """

    # Indicates fraud
    Fraud = 'FRAUD'
    # Indicates a false positive
    FalsePositive = 'FALSEPOSITIVE'
    # Indicates desired good is out of stock
    OutOfStock = 'OUTOFSTOCK'
    # Indicates desired good is in stock
    InStock = 'INSTOCK'
    # Indicates another reason
    Other = 'OTHER'
    # Indicates reason was not given
    NotGiven = 'NOTGIVEN'


class HppVersion(Enum):
    """
    Options when specifying HPP versions.
    Useful with `HostedPaymentConfig`.
    """

    #  HPP Version 1
    VERSION_1 = '1'
    #  HPP Version 2
    VERSION_2 = '2'


class FraudFilterMode(Enum):
    """
    Specify how the fraud filter should operate
    """

    #  Fraud filter will behave as configured in RealControl
    NONE = 0,
    #  Disables the fraud filter
    OFF = 1,
    #  Sets the fraud filter to passive mode
    PASSIVE = 2


class ReservationProviders(Enum):
    """
    Specifies the reservation service provider
    """
    FreshTxt = 1



"""
"""

from globalpayments.api import ServicesContainer
from globalpayments.api.builders.validations import Validations
from globalpayments.api.entities import (
    Address,
    TransactionReference,
)
from globalpayments.api.entities.enums import (
    AddressType,
    PaymentMethodType,
    ReportType,
    TransactionModifier,
    TransactionType,
)
from globalpayments.api.entities.exceptions import BuilderException
from globalpayments.api.payment_methods import EBTCardData


class BaseBuilder(object):
    validations = None

    def __init__(self):
        self.validations = Validations()
        self.setup_validations()

    def __getattr__(self, name):
        def wrapper(*args):
            if name[:5] == 'with_':
                self.set_property_if_exists(name[5:], args[0])
                return self

            return False

        return wrapper

    def execute(self, config_name=None):
        self.validations.validate(self)
        return None

    def setup_validations(self):
        pass

    def set_property_if_exists(self, *args):
        if hasattr(self, args[0]):
            setattr(self, args[0], args[1])
        else:
            raise BuilderException('Unknown property {}'.format(args[0]))


class TransactionBuilder(BaseBuilder):
    transaction_type = None
    transaction_modifier = None
    payment_method = None

    def __init__(self, transaction_type, payment_method=None):
        BaseBuilder.__init__(self)
        self.transaction_type = transaction_type
        self.payment_method = payment_method


class AuthorizationBuilder(TransactionBuilder):
    """
    Used to create charges, verifies, etc. for the supported
    payment method types.
    """

    account_type = None
    alias = None
    alias_action = None
    allow_duplicates = None
    allow_partial_auth = None
    amount = None
    auth_amount = None
    balance_inquiry_type = None
    billing_address = None
    cash_back_amount = None
    chip_condition = None
    client_transaction_id = None
    currency = None
    customer_id = None
    customer_ip_address = None
    cvn = None
    description = None
    dynamic_descriptor = None
    ecommerce_info = None
    gratuity = None
    convenience_amt = None
    shipping_amt = None
    hosted_payment_data = None
    invoice_number = None
    level_2_request = None
    message_authentication_code = None
    offline_auth_code = None
    one_time_payment = False
    order_id = None
    pos_sequence_number = None
    product_id = None
    recurring_sequence = None
    recurring_type = None
    request_multi_use_token = False
    replacement_card = None
    schedule_id = None
    shipping_address = None
    timestamp = None

    def with_address(self, address, address_type=AddressType.Billing):
        if not isinstance(address, Address):
            raise BuilderException('address must be of type Address')
        address.address_type = address_type
        if address_type is AddressType.Billing:
            self.billing_address = address
        else:
            self.shipping_address = address
        return self

    def with_alias(self, action, value):
        self.alias = value
        self.alias_action = action
        return self

    def with_cash_back(self, value):
        self.cash_back_amount = value
        self.transaction_modifier = TransactionModifier.CashBack
        return self

    def with_client_transaction_id(self, value):
        if (self.transaction_type is TransactionType.Reversal
                or self.transaction_type is TransactionType.Refund):
            if isinstance(self.payment_method, TransactionReference):
                self.payment_method.client_transaction_id = value
            else:
                self.payment_method = TransactionReference()
                self.payment_method.client_transaction_id = value
        else:
            self.client_transaction_id = value
        return self

    def with_commercial_request(self, value):
        self.level_2_request = value
        self.transaction_modifier = TransactionModifier.LevelII
        return self

    def with_hosted_payment_data(self, value):
        client = ServicesContainer.instance().get_client('default')
        if client.supports_hosted_payments:
            self.hosted_payment_data = value
            return self
        raise BuilderException(
            'Your current gateway does not support hosted payments.')

    def with_offline_auth_code(self, value):
        self.offline_auth_code = value
        self.transaction_modifier = TransactionModifier.Offline
        return self

    def with_one_time_payment(self, value):
        self.one_time_payment = value
        self.transaction_modifier = TransactionModifier.Recurring
        return self

    def with_payment_method(self, value):
        self.payment_method = value
        if isinstance(value, EBTCardData) and value.serial_number is not None:
            self.transaction_modifier = TransactionModifier.Voucher
        return self

    def with_recurring_info(self, recurring_type, recurring_sequence):
        self.recurring_type = recurring_type
        self.recurring_sequence = recurring_sequence
        return self

    def with_transaction_id(self, value):
        if isinstance(self.payment_method, TransactionReference):
            self.payment_method.transaction_id = value
        else:
            self.payment_method = TransactionReference()
            self.payment_method.transaction_id = value
        return self

    def __init__(self, transaction_type, payment_method=None):
        TransactionBuilder.__init__(self, transaction_type, payment_method)

    def execute(self, config_name=None):
        """
        Executes the authorization builder against the gateway.
        :return: Transaction
        """

        if config_name is None:
            config_name = "default"

        TransactionBuilder.execute(self)

        client = ServicesContainer.instance().get_client(config_name)
        return client.process_authorization(self)

    def serialize(self, config_name=None):
        """
        Serializes an authorization builder for hosted payment page requests.
        Requires the gateway and account support hosted payment pages.
        :return: string
        """

        if config_name is None:
            config_name = "default"

        self.transaction_modifier = TransactionModifier.HostedRequest
        TransactionBuilder.execute(self)

        client = ServicesContainer.instance().get_client(config_name)
        if client and client.supports_hosted_payments:
            return client.serialize_request(self)
        raise BuilderException(
            'Your current gateway does not support hosted payments.')

    def setup_validations(self):
        self.validations.of(TransactionType.Auth | TransactionType.Sale |
                            TransactionType.Refund | TransactionType.AddValue) \
            .check('amount').is_not_none() \
            .check('currency').is_not_none() \
            .check('payment_method').is_not_none()

        self.validations.of(TransactionType.Auth |
                            TransactionType.Sale |
                            TransactionType.Verify) \
            .with_constraint('transaction_modifier', TransactionModifier.HostedRequest) \
            .check('amount').is_not_none() \
            .check('currency').is_not_none()

        self.validations.of(TransactionType.Auth | TransactionType.Sale) \
            .with_constraint('transaction_modifier', TransactionModifier.Offline) \
            .check('amount').is_not_none() \
            .check('currency').is_not_none() \
            .check('offline_auth_code').is_not_none()

        self.validations.of(TransactionType.BenefitWithdrawal) \
            .with_constraint('transaction_modifier', TransactionModifier.CashBack) \
            .check('amount').is_not_none() \
            .check('currency').is_not_none() \
            .check('payment_method').is_not_none()

        self.validations.of(
            TransactionType.Balance).check('payment_method').is_not_none()

        self.validations.of(TransactionType.Alias) \
            .check('alias_action').is_not_none() \
            .check('alias').is_not_none()

        self.validations.of(
            TransactionType.Replace).check('replacement_card').is_not_none()

        self.validations.of(0) \
            .with_constraint('payment_method', PaymentMethodType.ACH) \
            .check('billing_address').is_not_none()


class ManagementBuilder(TransactionBuilder):
    """
    Used to follow up transactions for the supported
    payment method types.
    """

    amount = None
    auth_amount = None
    currency = None
    description = None
    gratuity = None
    po_number = None
    reason_code = None
    tax_amount = None
    tax_type = None

    @property
    def authorization_code(self):
        if isinstance(self.payment_method, TransactionReference):
            return self.payment_method.auth_code
        return None

    @property
    def client_transaction_id(self):
        if isinstance(self.payment_method, TransactionReference):
            return self.payment_method.client_transaction_id
        return None

    @property
    def order_id(self):
        if isinstance(self.payment_method, TransactionReference):
            return self.payment_method.order_id
        return None

    @property
    def transaction_id(self):
        if isinstance(self.payment_method, TransactionReference):
            return self.payment_method.transaction_id
        return None

    def with_po_number(self, value):
        self.transaction_modifier = TransactionModifier.LevelII
        self.po_number = value
        return self

    def with_tax_amount(self, value):
        self.transaction_modifier = TransactionModifier.LevelII
        self.tax_amount = value
        return self

    def with_tax_type(self, value):
        self.transaction_modifier = TransactionModifier.LevelII
        self.tax_type = value
        return self

    def __init__(self, transaction_type):
        TransactionBuilder.__init__(self, transaction_type)

    def execute(self, config_name=None):
        """
        Executes the builder against the gateway.
        :return: Transaction
        """

        if config_name is None:
            config_name = "default"

        TransactionBuilder.execute(self)

        client = ServicesContainer.instance().get_client(config_name)
        return client.manage_transaction(self)

    def setup_validations(self):
        self.validations.of(TransactionType.Capture | TransactionType.Edit |
                            TransactionType.Hold | TransactionType.Release) \
            .check('transaction_id').is_not_none()

        self.validations.of(TransactionType.Edit) \
            .with_constraint('transaction_modifier', TransactionModifier.LevelII) \
            .check('tax_type').is_not_none()

        self.validations.of(TransactionType.Refund) \
            .when('amount').is_not_none() \
            .check('currency').is_not_none()


class RecurringBuilder(TransactionBuilder):
    key = None
    order_id = None
    entity = None
    search_criteria = None

    def add_search_criteria(self, key, value):
        self.search_criteria[key] = value
        return self

    def __init__(self, transaction_type, entity=None):
        TransactionBuilder.__init__(self, transaction_type)
        self.entity = entity
        if entity is not None:
            self.key = entity.key
        self.search_criteria = {}

    def execute(self, config_name=None):
        """
        Executes the builder against the gateway.
        :return: RecurringEntity
        """

        if config_name is None:
            config_name = "default"

        TransactionBuilder.execute(self)

        client = ServicesContainer.instance().get_recurring_client(config_name)
        return client.process_recurring(self)

    def setup_validations(self):
        self.validations.of(TransactionType.Edit | TransactionType.Delete | TransactionType.Fetch) \
            .check('key').is_not_none()

        self.validations.of(
            TransactionType.Search).check('search_criteria').is_not_none()


class ReportBuilder(BaseBuilder):
    report_type = None
    timezone_conversion = None
    search_criteria = {}

    def add_search_criteria(self, key, value):
        self.search_criteria[key] = value
        return self

    def __init__(self, report_type):
        BaseBuilder.__init__(self)
        self.report_type = report_type

    def execute(self, config_name=None):
        """
        Executes the builder against the gateway.
        :return: Report
        """

        if config_name is None:
            config_name = "default"

        client = ServicesContainer.instance().get_client(config_name)
        return client.process_report(self)


class TransactionReportBuilder(ReportBuilder):
    device_id = None
    end_date = None
    start_date = None
    transaction_id = None

    def __init__(self, report_type):
        ReportBuilder.__init__(self, report_type)

    def setup_validations(self):
        self.validations.of(ReportType.TransactionDetail) \
            .check('transaction_id').is_not_none() \
            .check('device_id').is_none() \
            .check('end_date').is_none() \
            .check('start_date').is_none()

        self.validations.of(
            ReportType.Activity).check('transaction_id').is_none()



from globalpayments.api import ServicesContainer
from globalpayments.api.builders import TransactionBuilder
from globalpayments.api.entities.enums import TransactionModifier, TransactionType
from globalpayments.api.payment_methods import TransactionReference


class ManagementBuilder(TransactionBuilder):
    """
    Used to follow up transactions for the supported
    payment method types.
    """
    _amount = None
    _auth_amount = None
    _currency = None
    _description = None
    _gratuity = None
    _po_number = None
    _reason_code = None
    _tax_amount = None
    _tax_type = None

    @property
    def authorization_code(self):
        if isinstance(self._payment_method, TransactionReference):
            return self._payment_method.auth_code
        return None

    @property
    def client_transaction_id(self):
        if isinstance(self._payment_method, TransactionReference):
            return self._payment_method.client_transaction_id
        return None

    @property
    def order_id(self):
        if isinstance(self._payment_method, TransactionReference):
            return self._payment_method.order_id
        return None

    @property
    def transaction_id(self):
        if isinstance(self._payment_method, TransactionReference):
            return self._payment_method.transaction_id
        return None

    def with_po_number(self, value):
        self._transaction_modifier = TransactionModifier.LevelII
        self._po_number = value
        return self

    def with_tax_amount(self, value):
        self._transaction_modifier = TransactionModifier.LevelII
        self._tax_amount = value
        return self

    def with_tax_type(self, value):
        self._transaction_modifier = TransactionModifier.LevelII
        self._tax_type = value
        return self

    def __init__(self, transaction_type):
        TransactionBuilder.__init__(self, transaction_type)

    def execute(self):
        """
        Executes the builder against the gateway.
        :return: Transaction
        """
        TransactionBuilder.execute(self)

        client = ServicesContainer.instance().get_client()
        return client.manage_transaction(self)

    def setup_validations(self):
        self.validations.of(TransactionType.Capture | TransactionType.Edit |
                            TransactionType.Hold | TransactionType.Release) \
            .check('_transaction_id').is_not_none()

        self.validations.of(TransactionType.Edit).with_constraint(TransactionModifier.LevelII) \
            .check('_tax_type').is_not_none()

        self.validations.of(TransactionType.Refund) \
            .when('_amount').is_not_none() \
            .check('_currency').is_not_none()



import globalpayments as gp
from globalpayments.api.entities.enums import ReportType


class ReportingService(object):
    @staticmethod
    def activity():
        return gp.api.builders.TransactionReportBuilder(ReportType.Activity)

    @staticmethod
    def transaction_detail(transaction_id=None):
        return gp.api.builders.TransactionReportBuilder(ReportType.TransactionDetail) \
            .with_transaction_id(transaction_id)

    @staticmethod
    def find_transactions():
        return gp.api.builders.TransactionReportBuilder(ReportType.FindTransactions)



'''
ACH/eCheck payment method types
'''

import globalpayments as gp
from globalpayments.api.entities.enums import PaymentMethodType, TransactionType


class ECheck(object):
    '''
    Use ACH/eCheck as a payment method.
    '''

    account_number = None
    account_type = None
    ach_verify = None
    birth_year = None
    check_holder_name = None
    check_name = None
    check_number = None
    check_type = None
    check_verify = None
    drivers_license_number = None
    drivers_license_state = None
    entry_mode = None
    micr_number = None
    payment_method_type = None
    phone_number = None
    routing_number = None
    sec_code = None
    ssn_last_4 = None
    token = None

    payment_method_type = PaymentMethodType.ACH
    '''
    Set to L{PaymentMethodType.ACH} for internal methods.
    '''

    def charge(self, amount=None):
        '''
        Creates a charge (sale) against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount)



import globalpayments as gp
from globalpayments.api.entities.enums import AliasAction, PaymentMethodType, TransactionType
from globalpayments.api.entities.exceptions import ApiException


class GiftCard(object):
    payment_method_type = PaymentMethodType.Gift
    pin = None
    value_type = None
    value = None

    @property
    def alias(self):
        return self.value

    @alias.setter
    def alias(self, value):
        self.value = value
        self.value_type = 'Alias'

    @property
    def number(self):
        return self.value

    @number.setter
    def number(self, value):
        self.value = value
        self.value_type = 'CardNbr'

    @property
    def token(self):
        return self.value

    @token.setter
    def token(self, value):
        self.value = value
        self.value_type = 'TokenValue'

    @property
    def track_data(self):
        return self.value

    @track_data.setter
    def track_data(self, value):
        self.value = value
        self.value_type = 'TrackData'

    @staticmethod
    def create(alias=None, config_name='default'):
        card = GiftCard()

        try:
            response = gp.api.builders.AuthorizationBuilder(TransactionType.Alias, card) \
                .with_alias(AliasAction.Create, alias) \
                .execute(config_name)

            if response.response_code == '00':
                return response.gift_card

            raise ApiException(response.response_message)
        except Exception:
            raise ApiException('Unable to create gift card alias')

    def add_alias(self, alias=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Alias, self) \
            .with_alias(AliasAction.Add, alias)

    def activate(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Activate, self) \
            .with_amount(amount)

    def add_value(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.AddValue, self) \
            .with_amount(amount)

    def balance_inquiry(self, inquiry=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Balance, self) \
            .with_balance_inquiry_type(inquiry)

    def charge(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount)

    def deactivate(self):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Deactivate,
                                                    self)

    def remove_alias(self, alias=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Alias, self) \
            .with_alias(AliasAction.Delete, alias)

    def replace_with(self, new_card=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Replace, self) \
            .with_replacement_card(new_card)

    def reverse(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Reversal, self) \
            .with_amount(amount)

    def rewards(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Reward, self) \
            .with_amount(amount)



"""
Utilities for general use in the SDK
"""

import base64
from datetime import datetime
import hashlib
import uuid
from xml.dom import minidom


class ElementTree(object):
    """
    Wraps xml.dom.minidom to provide a simpler et
    """

    doc = None
    namespaces = {}

    def __init__(self):
        self.doc = minidom.Document()
        self.namespaces['soap'] = 'http://schemas.xmlsoap.org/soap/envelope/'

    def element(self, tag_name):
        """
        Creates a new element
        """

        element = self._create_element(tag_name)
        return Element(self.doc, element)

    def sub_element(self, parent, tag_name, **kwargs):
        """
        Creates a sub-element on a parent element
        """

        value = ''
        if 'value' in kwargs:
            if kwargs['value'] is None:
                return None
            value = kwargs['value']

        child = self._create_element(tag_name)
        parent.element.appendChild(child)
        return Element(self.doc, child).text(value)

    def to_string(self, root):
        """
        Converts an ElementTree object to its string form
        """

        self.doc.appendChild(root.element)
        try:
            return self.doc.toxml()
        finally:
            self.doc.removeChild(root)

    def get(self, tag_name):
        """
        Gets an element by tag_name
        """

        try:
            node = self.doc.getElementsByTagName(tag_name)[0]
            return Element(self.doc, node)
        except IndexError:
            return None

    def parse(self, source):
        """
        Converts a string or bytearray to an ElementTree
        """

        if isinstance(source, str):
            self.doc = minidom.parseString(source)
        elif isinstance(source, bytearray):
            self.doc = minidom.parseString(str(source))

    def _create_element(self, tag_name):
        if tag_name.contains(':'):
            data = tag_name.split(':')
            return self.doc.createElement(data[0], data[1],
                                          self.namespaces[data[0]])
        return self.doc.createElement(tag_name)


class Element(object):
    """
    Assists in working with ElementTree elements
    """

    doc = None
    element = None

    def __init__(self, doc, element):
        self.doc = doc
        self.element = element

    def first_child(self):
        """
        Gets the first child of the element
        """

        return self.from_node(self.doc, self.element.first_child())

    def prefix(self, prefix):
        """
        Sets the prefix of the element
        """

        self.element.prefix(prefix)
        return self

    def remove(self, tag_name):
        """
        Removes the child node by tag_name if it exists
        """

        child = self.get(tag_name)
        if child is not None:
            self.element.removeChild(tag_name)
        return self

    def set(self, name, value):
        """
        Sets an attribute by name on the element
        """

        self.element.setAttribute(name, value)
        return self

    def text(self, value):
        """
        Sets the inner text of the element
        """

        self.element.innerText(value)
        return self

    def append(self, child):
        """
        Adds a child to the element
        """

        self.doc.importNode(child)
        self.element.appendChild(child.element)
        return self

    def tag(self):
        """
        Gets the element's tag
        """

        return self.element.tag

    def has(self, tag_name):
        """
        Tests if a child exists with the tag_name
        """

        return len(self.element.getElementsByTagName(tag_name)) > 0

    def get(self, tag_name):
        """
        Get the first child that matches the tag_name
        """

        return self.element.getElementsByTagName(tag_name)[0]

    def get_all(self, tag_name):
        """
        Get all children that match the tag_name
        """

        return self.element.getElementsByTagName(tag_name)

    def get_value(self, *args):
        """
        Get the value of a child element
        """

        try:
            for tag_name in args:
                node = self.element.getElementsByTagName(tag_name)[0]
                if node is not None:
                    return node.innerText
        except IndexError:
            return None

    def get_attribute(self, attribute_name):
        """
        Get an element's attribute value
        """

        return self.element.getAttribute(attribute_name)

    @staticmethod
    def from_node(doc, node):
        """
        Helper method to create an Element object
        """

        return Element(doc, node)


class GenerationUtils(object):
    """
    Collection of generation tools
    """

    @staticmethod
    def generate_hash(shared_secret, to_hash=None):
        """
        Generates a Realex hash
        """

        if to_hash is None:
            return hashlib.sha1(shared_secret.encode()).hexdigest()

        first_pass = hashlib.sha1('.'.join(to_hash).encode()).hexdigest()
        return hashlib.sha1(
            (first_pass + '.' + shared_secret).encode()).hexdigest()

    @staticmethod
    def generate_order_id():
        """
        Generates a pseudo-random order id
        """

        return base64.b64encode(bytearray(str(uuid.uuid4()).encode())).decode() \
            .replace('=', '') \
            .replace('+', '-') \
            .replace('/', '_')

    @staticmethod
    def generate_recurring_key():
        """
        Generates a pseudo-random recurring key
        """

        return str(uuid.uuid4()).lower()

    @staticmethod
    def generate_timestamp():
        """
        Generates a timestamp in a Realex-compliant format (YYYYMMDDHHMMSS)
        """

        return datetime.now().strftime('%Y%m%d%H%M%S')



from datetime import datetime, timedelta
from globalpayments.api.entities.enums import ECommerceChannel


class ECommerceInfo(object):
    """
    ECommerce specific data to pass during authorization/settlement.
    """

    cavv = None
    channel = None
    eci = None
    payment_data_source = None
    payment_data_type = None
    ship_day = None
    ship_month = None
    xid = None

    def __init__(self):
        self.channel = ECommerceChannel.ECOM
        tomorrow = datetime.now() + timedelta(days=1)
        self.ship_day = tomorrow.day
        self.ship_month = tomorrow.month
        self.payment_data_type = '3DSecure'



import globalpayments.api
from globalpayments.api.builders.validations.validation_target import ValidationTarget


class Validations(object):
    rules = None

    def __init__(self):
        self.rules = {}

    def of(self, type_name):
        '''
        Creates a new `ValidationTarget` for the given
        transaction type mask.

        @type type_name: TransactionType
        @param type_name: Mask of transaction types
        @rtype: ValidationTarget
        @return: The current validation
        '''
        if type_name not in self.rules:
            self.rules[type_name] = []

        target = ValidationTarget(self, type_name)
        self.rules[type_name].append(target)
        return target

    def validate(self, builder):
        '''
        Validates a given builder with the defined validation rules

        :type builder: BaseBuilder
        :param builder: The builder
        '''

        for key in self.rules:
            value = self._get_property_value(builder, 'transaction_type')

            if value != key & value:
                continue

            for validation in self.rules[key]:
                if validation.clause is None:
                    continue

                #  modifier
                if validation.constraint_name is not None:
                    modifier = self._get_property_value(
                        builder, validation.constraint_name)
                    if validation.constraint_value is not modifier:
                        continue

                # check precondition
                if validation.precondition is not None:
                    if not validation.precondition.callback(builder):
                        continue

                if not validation.clause.callback(builder):
                    raise globalpayments.api.entities.exceptions.BuilderException(
                        validation.clause.message)

    @staticmethod
    def _get_property_value(obj, comp):
        if obj is None:
            return None

        try:
            return getattr(obj, comp)
        except AttributeError as _exc:
            return None



class ThreeDSecure(object):
    enrolled = None
    payer_authentication_response = None
    issuer_acs_url = None

    status = None
    eci = None
    xid = None
    cavv = None
    algorithm = None



import globalpayments as gp
from globalpayments.api.entities.enums import CvnPresenceIndicator, InquiryType, PaymentMethodType, TransactionType


class EBT(object):
    payment_method_type = PaymentMethodType.EBT
    pin_block = None

    @property
    def is_card_data(self):
        '''
        Helper method to test if a L{Credit} object is card data.
        '''
        try:
            _number_attr = getattr(self, 'number')
            return True
        except AttributeError as _exc:
            return False

    @property
    def is_track_data(self):
        '''
        Helper method to test if a L{Credit} object is track data.
        '''
        try:
            _number_attr = getattr(self, 'value')
            return True
        except AttributeError as _exc:
            return False

    def add_value(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.AddValue, self) \
            .with_amount(amount)

    def balance_inquiry(self, inquiry=InquiryType.FoodStamp):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Balance, self) \
            .with_balance_inquiry_type(inquiry) \
            .with_amount(0)

    def benefit_withdrawal(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.BenefitWithdrawal, self) \
            .with_amount(amount)

    def charge(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount)

    def refund(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Refund, self) \
            .with_amount(amount)

    def reverse(self, amount=None):
        raise NotImplementedError()


class EBTCardData(EBT):
    approval_code = None
    number = None
    exp_month = None
    exp_year = None
    cvn = None
    cvn_presence_indicator = CvnPresenceIndicator.NotRequested
    card_holder_name = None
    card_present = False
    reader_present = False
    serial_number = None


class EBTTrackData(EBT):
    encryption_data = None
    entry_method = None
    value = None



class EncryptionData(object):
    """
    Details how encrypted track data was encrypted by the device
    in order for the gateway to decrypt the data.
    """

    version = None
    track_number = None
    ksn = None
    ktb = None

    @staticmethod
    def version_1():
        rvalue = EncryptionData()
        rvalue.version = '01'
        return rvalue

    @staticmethod
    def version_2(ktb, track_number=None):
        rvalue = EncryptionData()
        rvalue.version = '02'
        rvalue.ktb = ktb
        rvalue.track_number = track_number
        return rvalue



# -*- coding: utf-8 -*-


class CountryUtils(object):
    significant_country_match = 6
    significant_code_match = 3

    country_code_map_by_country = dict()
    country_code_map_by_country["Afghanistan"] = "AF"
    country_code_map_by_country["Aland Islands"] = "AX"
    country_code_map_by_country["Albania"] = "AL"
    country_code_map_by_country["Algeria"] = "DZ"
    country_code_map_by_country["American Samoa"] = "AS"
    country_code_map_by_country["Andorra"] = "AD"
    country_code_map_by_country["Angola"] = "AO"
    country_code_map_by_country["Anguilla"] = "AI"
    country_code_map_by_country["Antarctica"] = "AQ"
    country_code_map_by_country["Antigua and Barbuda"] = "AG"
    country_code_map_by_country["Argentina"] = "AR"
    country_code_map_by_country["Armenia"] = "AM"
    country_code_map_by_country["Aruba"] = "AW"
    country_code_map_by_country["Australia"] = "AU"
    country_code_map_by_country["Austria"] = "AT"
    country_code_map_by_country["Azerbaijan"] = "AZ"
    country_code_map_by_country["Bahamas"] = "BS"
    country_code_map_by_country["Bahrain"] = "BH"
    country_code_map_by_country["Bangladesh"] = "BD"
    country_code_map_by_country["Barbados"] = "BB"
    country_code_map_by_country["Belarus"] = "BY"
    country_code_map_by_country["Belgium"] = "BE"
    country_code_map_by_country["Belize"] = "BZ"
    country_code_map_by_country["Benin"] = "BJ"
    country_code_map_by_country["Bermuda"] = "BM"
    country_code_map_by_country["Bhutan"] = "BT"
    country_code_map_by_country["Bolivia (Plurinational State of)"] = "BO"
    country_code_map_by_country["Bonaire, Sint Eustatius and Saba"] = "BQ"
    country_code_map_by_country["Bosnia and Herzegovina"] = "BA"
    country_code_map_by_country["Botswana"] = "BW"
    country_code_map_by_country["Bouvet Island"] = "BV"
    country_code_map_by_country["Brazil"] = "BR"
    country_code_map_by_country["British Indian Ocean Territory"] = "IO"
    country_code_map_by_country["Brunei Darussalam"] = "BN"
    country_code_map_by_country["Bulgaria"] = "BG"
    country_code_map_by_country["Burkina Faso"] = "BF"
    country_code_map_by_country["Burundi"] = "BI"
    country_code_map_by_country["Cambodia"] = "KH"
    country_code_map_by_country["Cameroon"] = "CM"
    country_code_map_by_country["Canada"] = "CA"
    country_code_map_by_country["Cabo Verde"] = "CV"
    country_code_map_by_country["Cayman Islands"] = "KY"
    country_code_map_by_country["Central African Republic"] = "CF"
    country_code_map_by_country["Chad"] = "TD"
    country_code_map_by_country["Chile"] = "CL"
    country_code_map_by_country["China"] = "CN"
    country_code_map_by_country["Christmas Island"] = "CX"
    country_code_map_by_country["Cocos (Keeling) Islands"] = "CC"
    country_code_map_by_country["Colombia"] = "CO"
    country_code_map_by_country["Comoros"] = "KM"
    country_code_map_by_country["Congo"] = "CG"
    country_code_map_by_country["Congo (Democratic Republic of the)"] = "CD"
    country_code_map_by_country["Cook Islands"] = "CK"
    country_code_map_by_country["Costa Rica"] = "CR"
    country_code_map_by_country["CÃ´te d'Ivoire"] = "CI"
    country_code_map_by_country["Croatia"] = "HR"
    country_code_map_by_country["Cuba"] = "CU"
    country_code_map_by_country["Curacao"] = "CW"
    country_code_map_by_country["Cyprus"] = "CY"
    country_code_map_by_country["Czech Republic"] = "CZ"
    country_code_map_by_country["Denmark"] = "DK"
    country_code_map_by_country["Djibouti"] = "DJ"
    country_code_map_by_country["Dominica"] = "DM"
    country_code_map_by_country["Dominican Republic"] = "DO"
    country_code_map_by_country["Ecuador"] = "EC"
    country_code_map_by_country["Egypt"] = "EG"
    country_code_map_by_country["El Salvador"] = "SV"
    country_code_map_by_country["Equatorial Guinea"] = "GQ"
    country_code_map_by_country["Eritrea"] = "ER"
    country_code_map_by_country["Estonia"] = "EE"
    country_code_map_by_country["Ethiopia"] = "ET"
    country_code_map_by_country["Falkland Islands (Malvinas)"] = "FK"
    country_code_map_by_country["Faroe Islands"] = "FO"
    country_code_map_by_country["Fiji"] = "FJ"
    country_code_map_by_country["Finland"] = "FI"
    country_code_map_by_country["France"] = "FR"
    country_code_map_by_country["French Guiana"] = "GF"
    country_code_map_by_country["French Polynesia"] = "PF"
    country_code_map_by_country["French Southern Territories"] = "TF"
    country_code_map_by_country["Gabon"] = "GA"
    country_code_map_by_country["Gambia"] = "GM"
    country_code_map_by_country["Georgia"] = "GE"
    country_code_map_by_country["Germany"] = "DE"
    country_code_map_by_country["Ghana"] = "GH"
    country_code_map_by_country["Gibraltar"] = "GI"
    country_code_map_by_country["Greece"] = "GR"
    country_code_map_by_country["Greenland"] = "GL"
    country_code_map_by_country["Grenada"] = "GD"
    country_code_map_by_country["Guadeloupe"] = "GP"
    country_code_map_by_country["Guam"] = "GU"
    country_code_map_by_country["Guatemala"] = "GT"
    country_code_map_by_country["Guernsey"] = "GG"
    country_code_map_by_country["Guinea"] = "GN"
    country_code_map_by_country["Guinea-Bissau"] = "GW"
    country_code_map_by_country["Guyana"] = "GY"
    country_code_map_by_country["Haiti"] = "HT"
    country_code_map_by_country["Heard Island and McDonald Islands"] = "HM"
    country_code_map_by_country["Holy See"] = "VA"
    country_code_map_by_country["Honduras"] = "HN"
    country_code_map_by_country["Hong Kong"] = "HK"
    country_code_map_by_country["Hungary"] = "HU"
    country_code_map_by_country["Iceland"] = "IS"
    country_code_map_by_country["India"] = "IN"
    country_code_map_by_country["Indonesia"] = "ID"
    country_code_map_by_country["Iran (Islamic Republic of)"] = "IR"
    country_code_map_by_country["Iraq"] = "IQ"
    country_code_map_by_country["Ireland"] = "IE"
    country_code_map_by_country["Isle of Man"] = "IM"
    country_code_map_by_country["Israel"] = "IL"
    country_code_map_by_country["Italy"] = "IT"
    country_code_map_by_country["Jamaica"] = "JM"
    country_code_map_by_country["Japan"] = "JP"
    country_code_map_by_country["Jersey"] = "JE"
    country_code_map_by_country["Jordan"] = "JO"
    country_code_map_by_country["Kazakhstan"] = "KZ"
    country_code_map_by_country["Kenya"] = "KE"
    country_code_map_by_country["Kiribati"] = "KI"
    country_code_map_by_country[
        "Korea (Democratic People's Republic of)"] = "KP"
    country_code_map_by_country["Korea (Republic of)"] = "KR"
    country_code_map_by_country["Kuwait"] = "KW"
    country_code_map_by_country["Kyrgyzstan"] = "KG"
    country_code_map_by_country["Lao People's Democratic Republic"] = "LA"
    country_code_map_by_country["Latvia"] = "LV"
    country_code_map_by_country["Lebanon"] = "LB"
    country_code_map_by_country["Lesotho"] = "LS"
    country_code_map_by_country["Liberia"] = "LR"
    country_code_map_by_country["Libya"] = "LY"
    country_code_map_by_country["Liechtenstein"] = "LI"
    country_code_map_by_country["Lithuania"] = "LT"
    country_code_map_by_country["Luxembourg"] = "LU"
    country_code_map_by_country["Macao"] = "MO"
    country_code_map_by_country[
        "Macedonia (the former Yugoslav Republic of)"] = "MK"
    country_code_map_by_country["Madagascar"] = "MG"
    country_code_map_by_country["Malawi"] = "MW"
    country_code_map_by_country["Malaysia"] = "MY"
    country_code_map_by_country["Maldives"] = "MV"
    country_code_map_by_country["Mali"] = "ML"
    country_code_map_by_country["Malta"] = "MT"
    country_code_map_by_country["Marshall Islands"] = "MH"
    country_code_map_by_country["Martinique"] = "MQ"
    country_code_map_by_country["Mauritania"] = "MR"
    country_code_map_by_country["Mauritius"] = "MU"
    country_code_map_by_country["Mayotte"] = "YT"
    country_code_map_by_country["Mexico"] = "MX"
    country_code_map_by_country["Micronesia (Federated States of)"] = "FM"
    country_code_map_by_country["Moldova (Republic of)"] = "MD"
    country_code_map_by_country["Monaco"] = "MC"
    country_code_map_by_country["Mongolia"] = "MN"
    country_code_map_by_country["Montenegro"] = "ME"
    country_code_map_by_country["Montserrat"] = "MS"
    country_code_map_by_country["Morocco"] = "MA"
    country_code_map_by_country["Mozambique"] = "MZ"
    country_code_map_by_country["Myanmar"] = "MM"
    country_code_map_by_country["Namibia"] = "NA"
    country_code_map_by_country["Nauru"] = "NR"
    country_code_map_by_country["Nepal"] = "NP"
    country_code_map_by_country["Netherlands"] = "NL"
    country_code_map_by_country["New Caledonia"] = "NC"
    country_code_map_by_country["New Zealand"] = "NZ"
    country_code_map_by_country["Nicaragua"] = "NI"
    country_code_map_by_country["Niger"] = "NE"
    country_code_map_by_country["Nigeria"] = "NG"
    country_code_map_by_country["Niue"] = "NU"
    country_code_map_by_country["Norfolk Island"] = "NF"
    country_code_map_by_country["Northern Mariana Islands"] = "MP"
    country_code_map_by_country["Norway"] = "NO"
    country_code_map_by_country["Oman"] = "OM"
    country_code_map_by_country["Pakistan"] = "PK"
    country_code_map_by_country["Palau"] = "PW"
    country_code_map_by_country["Palestine, State of"] = "PS"
    country_code_map_by_country["Panama"] = "PA"
    country_code_map_by_country["Papua New Guinea"] = "PG"
    country_code_map_by_country["Paraguay"] = "PY"
    country_code_map_by_country["Peru"] = "PE"
    country_code_map_by_country["Philippines"] = "PH"
    country_code_map_by_country["Pitcairn"] = "PN"
    country_code_map_by_country["Poland"] = "PL"
    country_code_map_by_country["Portugal"] = "PT"
    country_code_map_by_country["Puerto Rico"] = "PR"
    country_code_map_by_country["Qatar"] = "QA"
    country_code_map_by_country["RÃ©union"] = "RE"
    country_code_map_by_country["Romania"] = "RO"
    country_code_map_by_country["Russian Federation"] = "RU"
    country_code_map_by_country["Rwanda"] = "RW"
    country_code_map_by_country["Saint BarthÃ©lemy"] = "BL"
    country_code_map_by_country[
        "Saint Helena, Ascension and Tristan da Cunha"] = "SH"
    country_code_map_by_country["Saint Kitts and Nevis"] = "KN"
    country_code_map_by_country["Saint Lucia"] = "LC"
    country_code_map_by_country["Saint Martin (French part)"] = "MF"
    country_code_map_by_country["Saint Pierre and Miquelon"] = "PM"
    country_code_map_by_country["Saint Vincent and the Grenadines"] = "VC"
    country_code_map_by_country["Samoa"] = "WS"
    country_code_map_by_country["San Marino"] = "SM"
    country_code_map_by_country["Sao Tome and Principe"] = "ST"
    country_code_map_by_country["Saudi Arabia"] = "SA"
    country_code_map_by_country["Senegal"] = "SN"
    country_code_map_by_country["Serbia"] = "RS"
    country_code_map_by_country["Seychelles"] = "SC"
    country_code_map_by_country["Sierra Leone"] = "SL"
    country_code_map_by_country["Singapore"] = "SG"
    country_code_map_by_country["Sint Maarten (Dutch part)"] = "SX"
    country_code_map_by_country["Slovakia"] = "SK"
    country_code_map_by_country["Slovenia"] = "SI"
    country_code_map_by_country["Solomon Islands"] = "SB"
    country_code_map_by_country["Somalia"] = "SO"
    country_code_map_by_country["South Africa"] = "ZA"
    country_code_map_by_country[
        "South Georgia and the South Sandwich Islands"] = "GS"
    country_code_map_by_country["South Sudan"] = "SS"
    country_code_map_by_country["Spain"] = "ES"
    country_code_map_by_country["Sri Lanka"] = "LK"
    country_code_map_by_country["Sudan"] = "SD"
    country_code_map_by_country["Suriname"] = "SR"
    country_code_map_by_country["Svalbard and Jan Mayen"] = "SJ"
    country_code_map_by_country["Swaziland"] = "SZ"
    country_code_map_by_country["Sweden"] = "SE"
    country_code_map_by_country["Switzerland"] = "CH"
    country_code_map_by_country["Syrian Arab Republic"] = "SY"
    country_code_map_by_country["Taiwan, Province of China"] = "TW"
    country_code_map_by_country["Tajikistan"] = "TJ"
    country_code_map_by_country["Tanzania, United Republic of"] = "TZ"
    country_code_map_by_country["Thailand"] = "TH"
    country_code_map_by_country["Timor-Leste"] = "TL"
    country_code_map_by_country["Togo"] = "TG"
    country_code_map_by_country["Tokelau"] = "TK"
    country_code_map_by_country["Tonga"] = "TO"
    country_code_map_by_country["Trinidad and Tobago"] = "TT"
    country_code_map_by_country["Tunisia"] = "TN"
    country_code_map_by_country["Turkey"] = "TR"
    country_code_map_by_country["Turkmenistan"] = "TM"
    country_code_map_by_country["Turks and Caicos Islands"] = "TC"
    country_code_map_by_country["Tuvalu"] = "TV"
    country_code_map_by_country["Uganda"] = "UG"
    country_code_map_by_country["Ukraine"] = "UA"
    country_code_map_by_country["United Arab Emirates"] = "AE"
    country_code_map_by_country[
        "United Kingdom of Great Britain and Northern Ireland"] = "GB"
    country_code_map_by_country["United States of America"] = "US"
    country_code_map_by_country["United States Minor Outlying Islands"] = "UM"
    country_code_map_by_country["Uruguay"] = "UY"
    country_code_map_by_country["Uzbekistan"] = "UZ"
    country_code_map_by_country["Vanuatu"] = "VU"
    country_code_map_by_country["Venezuela (Bolivarian Republic of)"] = "VE"
    country_code_map_by_country["Viet Nam"] = "VN"
    country_code_map_by_country["Virgin Islands (British)"] = "VG"
    country_code_map_by_country["Virgin Islands (U.S.)"] = "VI"
    country_code_map_by_country["Wallis and Futuna"] = "WF"
    country_code_map_by_country["Western Sahara"] = "EH"
    country_code_map_by_country["Yemen"] = "YE"
    country_code_map_by_country["Zambia"] = "ZM"
    country_code_map_by_country["Zimbabwe"] = "ZW"

    country_map_by_country_code = dict()
    for country in country_code_map_by_country.keys():
        country_map_by_country_code[country_code_map_by_country[
            country]] = country

    @staticmethod
    def is_country(address, country_code):
        if address.country_code is not None:
            return address.country_code == country_code

        if address.country is not None:
            code = CountryUtils.get_country_code_by_country(address.country)

            if code is not None:
                return code == country_code

            return False

        return False

    @staticmethod
    def get_country_by_country_code(country_code):
        if country_code is None:
            return None

        if country_code in CountryUtils.country_map_by_country_code:
            return CountryUtils.country_map_by_country_code[country_code]

        if len(country_code) > 3:
            return None

        return CountryUtils.fuzzy_match(
            CountryUtils.country_map_by_country_code, country_code,
            CountryUtils.significant_code_match)

    @staticmethod
    def get_country_code_by_country(country):
        if country is None:
            return None

        if country in CountryUtils.country_code_map_by_country:
            return CountryUtils.country_code_map_by_country[country]

        match = CountryUtils.fuzzy_match(
            CountryUtils.country_code_map_by_country, country,
            CountryUtils.significant_country_match)

        if match is not None:
            return match

        if len(country) > 3:
            return None

        match = CountryUtils.fuzzy_match(
            CountryUtils.country_map_by_country_code, country,
            CountryUtils.significant_code_match)

        if match is not None:
            return match

        return None

    @staticmethod
    def fuzzy_match(dictionary, query, significant_match):
        rvalue = None
        matches = {}

        # loop
        high_score = -1
        for key in dictionary.keys():
            score = CountryUtils.fuzzy_score(key, query)

            if score > significant_match and score > high_score:
                matches = {}
                high_score = score
                rvalue = dictionary[key]
                matches[key] = rvalue

            elif score == high_score:
                matches[key] = dictionary[key]

        if len(matches) > 1:
            return None

        return rvalue

    @staticmethod
    def fuzzy_score(term, query):
        if term is None or query is None:
            return None

        term_lower_case = term.lower()
        query_lower_case = query.lower()

        score = 0
        term_index = 0
        previous_matching_character_index = -1

        for query_index in range(len(query_lower_case)):
            query_char = query_lower_case[query_index]

            term_character_match_found = False
            for term_index in range(len(term_lower_case)):
                if term_character_match_found:
                    pass

                term_char = term_lower_case[term_index]

                if query_char == term_char:
                    score = score + 1

                    if previous_matching_character_index + 1 == term_index:
                        score = score + 2

                    previous_matching_character_index = term_index
                    term_character_match_found = True

        return score


class Address(object):
    """
    Represents a billing or shipping address for the consumer.
    """

    address_type = None
    street_address_1 = None
    street_address_2 = None
    street_address_3 = None
    city = None
    province = None
    postal_code = None
    _country = None
    _country_code = None

    @property
    def state(self):
        return self.province

    @state.setter
    def state(self, value):
        self.province = value

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._country = value
        if self._country_code is None:
            self._country_code = CountryUtils.get_country_code_by_country(
                self._country)

    @property
    def country_code(self):
        return self._country_code

    @country_code.setter
    def country_code(self, value):
        self._country_code = value
        if self._country is None:
            self._country = CountryUtils.get_country_by_country_code(
                self._country_code)



"""
"""

import globalpayments as gp
from globalpayments.api.entities.address import Address
from globalpayments.api.entities.batch_summary import BatchSummary
from globalpayments.api.entities.debit_mac import DebitMac
from globalpayments.api.entities.ecommerce_info import ECommerceInfo
from globalpayments.api.entities.encryption_data import EncryptionData
from globalpayments.api.entities.enums import PaymentMethodType, TransactionType
from globalpayments.api.entities.exceptions import ApiException, UnsupportedTransactionException
from globalpayments.api.entities.three_d_secure import ThreeDSecure
from globalpayments.api.entities.transaction_summary import TransactionSummary
from globalpayments.api.payment_methods import TransactionReference
from globalpayments.api.services import RecurringService


class Transaction(object):
    """
    Transaction Response
    """
    authorized_amount = None
    available_balance = None
    avs_response_code = None
    avs_response_message = None
    balance_amount = None
    batch_summary = None
    card_type = None
    card_last_4 = None
    cavv_response_code = None
    commercial_indicator = None
    cvn_response_code = None
    cvn_response_message = None
    emv_issuer_response = None
    points_balance_amount = None
    recurring_data_code = None
    reference_number = None
    response_code = None
    response_message = None
    response_values = {}
    timestamp = None
    transaction_descriptor = None
    token = None
    gift_card = None
    transaction_reference = None

    @property
    def authorization_code(self):
        if self.transaction_reference is not None:
            return self.transaction_reference.auth_code
        return None

    @authorization_code.setter
    def authorization_code(self, value):
        if self.transaction_reference is None:
            self.transaction_reference = TransactionReference()
        self.transaction_reference.auth_code = value

    @property
    def client_transaction_id(self):
        if self.transaction_reference is not None:
            return self.transaction_reference.client_transaction_id
        return None

    @client_transaction_id.setter
    def client_transaction_id(self, value):
        if self.transaction_reference is None:
            self.transaction_reference = TransactionReference()
        self.transaction_reference.client_transaction_id = value

    @property
    def order_id(self):
        if self.transaction_reference is not None:
            return self.transaction_reference.order_id
        return None

    @order_id.setter
    def order_id(self, value):
        if self.transaction_reference is None:
            self.transaction_reference = TransactionReference()
        self.transaction_reference.order_id = value

    @property
    def payment_method_type(self):
        if self.transaction_reference is not None:
            return self.transaction_reference.payment_method_type
        return None

    @payment_method_type.setter
    def payment_method_type(self, value):
        if self.transaction_reference is None:
            self.transaction_reference = TransactionReference()
        self.transaction_reference.payment_method_type = value

    @property
    def transaction_id(self):
        if self.transaction_reference is not None:
            return self.transaction_reference.transaction_id
        return None

    @transaction_id.setter
    def transaction_id(self, value):
        if self.transaction_reference is None:
            self.transaction_reference = TransactionReference()
        self.transaction_reference.transaction_id = value

    @staticmethod
    def from_id(transaction_id,
                payment_method_type=PaymentMethodType.Credit,
                order_id=None):
        """
        Creates a `Transaction` object from a stored transaction ID.
        Used to expose management requests on the original transaction
        at a later date/time.
        :param transaction_id: The original transaction ID
        :param payment_method_type: The original payment method type.
            Defaults to `PaymentMethodType.Credit`.
        :param order_id: The original transaction's order ID
        :return: Transaction
        """
        rvalue = Transaction()
        rvalue.transaction_id = transaction_id
        rvalue.payment_method_type = payment_method_type
        rvalue.order_id = order_id
        return rvalue

    def additional_auth(self, amount=None):
        """
        Creates an additional authorization against the original transaction.
        :param amount: The additional amount to authorize
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Auth) \
            .with_payment_method(self.transaction_reference) \
            .with_amount(amount)

    def capture(self, amount=None):
        """
        Captures the original transaction.
        :param amount: The amount to capture
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Capture) \
            .with_payment_method(self.transaction_reference) \
            .with_amount(amount)

    def edit(self):
        """
        Edits the original transaction.
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Edit) \
            .with_payment_method(self.transaction_reference)

    def hold(self):
        """
        Places the original transaction on hold.
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Hold) \
            .with_payment_method(self.transaction_reference)

    def refund(self, amount=None):
        """
        Refunds/returns the original transaction.
        :param amount:The amount to refund/return
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Refund) \
            .with_payment_method(self.transaction_reference) \
            .with_amount(amount)

    def release(self):
        """
        Releases the original transaction from a hold.
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Release) \
            .with_payment_method(self.transaction_reference)

    def reverse(self, amount=None):
        """
        Reverses the original transaction.
        :param amount: The original authorization amount
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Reversal) \
            .with_payment_method(self.transaction_reference) \
            .with_amount(amount)

    def void(self):
        """
        Voids the original transaction.
        :return: ManagementBuilder
        """
        return gp.api.builders.ManagementBuilder(TransactionType.Void) \
            .with_payment_method(self.transaction_reference)


class RecurringEntity(object):
    """
    Base implementation for recurring resource types.
    """

    id = None
    key = None

    def create(self, config_name = 'default'):
        """
        Creates a resource
        :return: RecurringEntity
        """

        return gp.api.services.RecurringService.create(self, config_name)

    def delete(self, force=False, config_name = 'default'):
        """
        Delete a record from the gateway.
        :param force: Indicates if the deletion should be forced
        :return: void
        """
        try:
            return gp.api.services.RecurringService.delete(
                self, force, config_name)
        except ApiException as exc:
            raise ApiException(
                'Failed to delete record, see inner exception for more details. ',
                exc)

    @staticmethod
    def find(identifier_name, identifier, config_name = 'default'):
        """
        Searches for a specific record by `id`.
        :param identifier: The ID of the record to find
        :return: RecurringEntity or None if not found
        """
        client = gp.api.ServicesContainer.instance().get_recurring_client(config_name)
        if client is not None and client.supports_retrieval:
            response = gp.api.services.RecurringService.search() \
                .add_search_criteria(identifier_name, identifier) \
                .execute(config_name)
            entity = response[0] if len(response) > 0 else None 
            if entity is not None:
                return gp.api.services.RecurringService.get(
                    entity, config_name)
            return None
        raise UnsupportedTransactionException()

    @staticmethod
    def find_all(entity, config_name = 'default'):
        """
        Lists all records of base type
        :return: Array
        """
        client = gp.api.ServicesContainer.instance().get_recurring_client(
            config_name)
        if client is not None and client.supports_retrieval:
            return gp.api.services.RecurringService.search(entity).execute(
                config_name)
        raise UnsupportedTransactionException()

    def save_changes(self, config_name = 'default'):
        try:
            return gp.api.services.RecurringService.edit(self, config_name)
        except ApiException as exc:
            raise ApiException(
                'Update failed, see inner exception for more details. ' +
                exc.message, exc)


class Customer(RecurringEntity):
    """
    A customer resource.
    Mostly used in recurring scenarios.
    """

    title = None
    first_name = None
    last_name = None
    company = None
    address = None
    home_phone = None
    work_phone = None
    fax = None
    mobile_phone = None
    email = None
    comments = None
    department = None
    status = None
    payment_methods = None

    def add_payment_method(self, payment_id, payment_method):
        """
        Adds a payment method to the customer
        :param payment_id: An application derived ID for the payment method
        :param payment_method: The payment method
        :return: RecurringPaymentMethod
        """

        name_on_account = ""  # "%s %s".format(self.first_name, self.last_name)
        if not name_on_account:
            name_on_account = self.company

        method = RecurringPaymentMethod(payment_method)
        method.address = self.address
        method.customer_key = self.key
        method.id = payment_id
        method.name_on_account = name_on_account
        return method

    @staticmethod
    def find(identifier, config_name = 'default'):
        test = RecurringEntity.find('customerIdentifier', identifier, config_name)
        return test

    @staticmethod
    def find_all(config_name = 'default'):
        entity = Customer();
        return RecurringEntity.find_all(entity, config_name)

class RecurringPaymentMethod(RecurringEntity):
    address = None
    commercial_indicator = None
    customer_key = None
    expiration_date = None
    name_on_account = None
    payment_method = None
    payment_method_type = PaymentMethodType.Recurring
    payment_type = None
    preferred_payment = None
    status = None
    tax_type = None
    sec_code = None

    def __init__(self, payment_method_or_customer=None, payment_id=None):
        if isinstance(payment_method_or_customer, str):
            self.customer_key = payment_method_or_customer
            self.key = payment_id
            self.payment_type = "Credit Card"
        else:
            self.payment_method = payment_method_or_customer

    def authorize(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Auth, self) \
            .with_amount(amount) \
            .with_one_time_payment(True)

    def charge(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount) \
            .with_one_time_payment(True)

    def refund(self, amount=None):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Refund, self) \
            .with_amount(amount)

    def verify(self):
        return gp.api.builders.AuthorizationBuilder(TransactionType.Verify,
                                                    self)

    def add_schedule(self, schedule_id):
        data = Schedule()
        data.customer_key = self.customer_key
        data.payment_key = self.key
        data.id = schedule_id
        return data

    @staticmethod
    def find(identifier, config_name = 'default'):
        return RecurringEntity.find('paymentMethodIdentifier', identifier, config_name)

    @staticmethod
    def find_all(config_name = 'default'):
        entity = RecurringPaymentMethod();
        return RecurringEntity.find_all(entity, config_name)

class Schedule(RecurringEntity):
    amount = None
    cancellation_date = None
    currency = None
    customer_key = None
    description = None
    device_id = None
    email_notification = None
    email_receipt = None
    end_date = None
    frequency = None
    has_started = False
    invoice_number = None
    name = None
    next_processing_date = None
    number_of_payments = None
    po_number = None
    payment_key = None
    payment_schedule = None
    reprocessing_count = None
    start_date = None
    status = None
    tax_amount = None

    @property
    def total_amount(self):
        return self.amount + self.tax_amount

    def with_status(self, value):
        self.status = value
        return self

    def with_amount(self, value):
        self.amount = value * 100
        return self

    def with_reprocessing_count(self, value):
        self.reprocessing_count = value
        return self

    def with_start_date(self, value):
        self.start_date = value
        return self

    def with_end_date(self, value):
        self.end_date = value
        return self

    def with_frequency(self, value):
        self.frequency= value
        return self

    def with_currency(self, value):
        self.currency= value
        return self

    def with_email_receipt(self, value):
        self.email_receipt = value
        return self

    def __init__(self, customer_key=None, payment_key=None):
        self.customer_key = customer_key
        self.payment_key = payment_key

    @staticmethod
    def find(identifier, config_name = 'default'):
        return RecurringEntity.find('scheduleIdentifier', identifier, config_name)

    @staticmethod
    def find_all(config_name = 'default'):
        entity = Schedule();
        return RecurringEntity.find_all(entity, config_name)



import globalpayments.api
from globalpayments.api.builders.validations.validation_clause import ValidationClause


class ValidationTarget(object):
    parent = None
    precondition = None
    clause = None

    type_name = None
    constraint_name = None
    constraint_value = None

    def __init__(self, parent, type_name):
        self.parent = parent
        self.type_name = type_name

    def with_constraint(self, name, value):
        self.constraint_name = name
        self.constraint_value = value
        return self

    def check(self, property_name):
        if property_name is None:
            raise globalpayments.api.entities.exceptions.BuilderException()

        self.clause = ValidationClause(self.parent, self, property_name)
        return self.clause

    def when(self, property_name):
        if property_name is None:
            raise globalpayments.api.entities.exceptions.BuilderException()

        self.precondition = ValidationClause(self.parent, self, property_name,
                                             True)
        return self.precondition



import globalpayments as gp
from globalpayments.api.entities.enums import TransactionType


class RecurringService(object):
    @staticmethod
    def create(entity, config_name=None):
        return gp.api.builders.RecurringBuilder(TransactionType.Create,
                                                entity).execute(config_name)

    @staticmethod
    def delete(entity, force=False, config_name=None):
        return gp.api.builders.RecurringBuilder(TransactionType.Delete, entity) \
            .with_force(force) \
            .execute(config_name)

    @staticmethod
    def edit(entity, config_name=None):
        return gp.api.builders.RecurringBuilder(TransactionType.Edit,
                                                entity).execute(config_name)

    @staticmethod
    def get(entity, config_name=None):
        return gp.api.builders.RecurringBuilder(TransactionType.Fetch,
                                                entity).execute(config_name)

    @staticmethod
    def search(entity = None):
        return gp.api.builders.RecurringBuilder(TransactionType.Search, entity)



'''
Credit payment method types
'''

import re
import globalpayments as gp
from globalpayments.api.entities.enums import (
    CvnPresenceIndicator, PaymentMethodType, TransactionType)


class DebitTrackData(object):
    encryption_data = None
    payment_method_type = PaymentMethodType.Debit
    pin_block = None
    value = None

    @property
    def is_track_data(self):
        '''
        Helper method to test if a L{Credit} object is track data.
        '''
        try:
            _number_attr = getattr(self, 'value')
            return True
        except AttributeError as _exc:
            return False

    def add_value(self, amount=None):
        '''
        Adds value to to a payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.AddValue, self) \
            .with_amount(amount)

    def charge(self, amount=None):
        '''
        Creates a charge (sale) against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        order_id = None

        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount)

    def refund(self, amount=None):
        '''
        Refunds the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Refund, self) \
            .with_amount(amount)

    def reverse(self, amount=None):
        '''
        Reverses a previous transaction against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Reversal, self) \
            .with_amount(amount)



'''
Credit payment method types
'''

import re
import globalpayments as gp
from globalpayments.api.entities.enums import (
    CvnPresenceIndicator, PaymentMethodType, TransactionType)


class Credit(object):
    '''
    Use credit as a payment method.
    '''

    tokenizable = True

    encryption_data = None
    '''
    The card's encryption data; where applicable.
    '''

    payment_method_type = PaymentMethodType.Credit
    '''
    Set to L{PaymentMethodType.Credit} for internal methods.
    '''

    three_d_secure = None
    '''
    3DSecure data attached to the card
    '''

    token = None
    '''
    A token value representing the card.
    '''

    @property
    def is_card_data(self):
        '''
        Helper method to test if a L{Credit} object is card data.
        '''
        try:
            _number_attr = getattr(self, 'number')
            return True
        except AttributeError as _exc:
            return False

    @property
    def is_track_data(self):
        '''
        Helper method to test if a L{Credit} object is track data.
        '''
        try:
            _number_attr = getattr(self, 'value')
            return True
        except AttributeError as _exc:
            return False

    def add_value(self, amount=None):
        '''
        Adds value to to a payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.AddValue, self) \
            .with_amount(amount)

    def authorize(self, amount=None):
        '''
        Creates an authorization against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        currency = None
        order_id = None
        if self.three_d_secure is not None:
            currency = self.three_d_secure.currency
            order_id = self.three_d_secure.order_id

            if amount is None:
                amount = self.three_d_secure.amount

        return gp.api.builders.AuthorizationBuilder(TransactionType.Auth, self) \
            .with_amount(amount) \
            .with_currency(currency) \
            .with_order_id(order_id)

    def charge(self, amount=None):
        '''
        Creates a charge (sale) against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        currency = None
        order_id = None
        if self.three_d_secure is not None:
            currency = self.three_d_secure.currency
            order_id = self.three_d_secure.order_id

            if amount is None:
                amount = self.three_d_secure.amount

        return gp.api.builders.AuthorizationBuilder(TransactionType.Sale, self) \
            .with_amount(amount) \
            .with_currency(currency) \
            .with_order_id(order_id)

    def balance_inquiry(self, inquiry=None):
        '''
        Completes a balance inquiry (lookup) on the payment method.

        @type inquiry: L{BalanceInquiryType}
        @param inquiry: The type of inquiry to make
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Balance, self) \
            .with_balance_inquiry_type(inquiry)

    def refund(self, amount=None):
        '''
        Refunds the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Refund, self) \
            .with_amount(amount)

    def reverse(self, amount=None):
        '''
        Reverses a previous transaction against the payment method.

        @type amount: number
        @param amount: The amount of the transaction
        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Reversal, self) \
            .with_amount(amount)

    def tokenize(self):
        '''
        Tokenizes the payment method, verifying the payment method
        with the issuer in the process.

        @rtype: string
        @return: The requested token
        '''

        response = self.verify() \
            .with_request_multi_use_token(True) \
            .execute()
        return response.token

    def verify(self):
        '''
        Verifies the payment method with the issuer.

        @rtype: L{AuthorizationBuilder}
        @return: The builder
        '''

        return gp.api.builders.AuthorizationBuilder(TransactionType.Verify,
                                                    self)


class CreditCardData(Credit):
    '''
    Use credit tokens or manual entry data as a payment method.
    '''

    _cvn = None
    _number = None
    _regexDict = {
        'Amex': r'^3[47][0-9]{13}$',
        'MC': r'^5[1-5][0-9]{14}$',
        'Visa': r'^4[0-9]{12}(?:[0-9]{3})?$',
        'DinersClub': r'^3(?:0[0-5]|[68][0-9])[0-9]{11}$',
        'EnRoute': r'^(2014|2149)',
        'Discover': r'^6(?:011|5[0-9]{2})[0-9]{12}$',
        'Jcb': r'^(?:2131|1800|35\d{3})\d{11}$',
    }

    card_type = 'Unknown'
    '''
    The card type of the manual entry data.

    Default value is `Unknown`.
    '''

    card_present = False
    '''
    Indicates if the card is present with the merchant at time of payment.

    Default value is `False`.
    '''

    card_holder_name = None
    '''
    The name on the front of the card.
    '''

    cvn_presence_indicator = CvnPresenceIndicator.NotRequested
    '''
    Indicates card verification number (CVN) presence.

    Default value is L{CvnPresenceIndicator.NotRequested}.
    '''

    exp_month = None
    '''
    The card's expiration month.
    '''

    exp_year = None
    '''
    The card's expiration year.
    '''

    reader_present = False
    '''
    Indicates if a card reader was used when accepting the card data.

    Default value is `False`.
    '''

    @property
    def cvn(self):
        '''
        The card's card verification number (CVN).

        When set, L{CreditCardData.cvn_presence_indicator} is set to
        L{CvnPresenceIndicator.Present}.
        '''

        return self._cvn

    @cvn.setter
    def cvn(self, value):
        if value is not None and value != '':
            self._cvn = value
            self.cvn_presence_indicator = CvnPresenceIndicator.Present

    @property
    def number(self):
        '''
        The card's number.
        '''

        return self._number

    @number.setter
    def number(self, value):
        if value is None:
            return

        self._number = value.replace(' ', '').replace('-', '')

        for name in self._regexDict:
            if re.match(self._regexDict[name], self._number) is not None:
                self.card_type = name
                break

    @property
    def short_expiry(self):
        '''
        The card's expiration date in `MMYY` format.
        '''

        month = str(self.exp_month).zfill(2)
        year = str(self.exp_year).zfill(4)[2:]
        return '{}{}'.format(month, year)


class CreditTrackData(Credit):
    '''
    Use credit track data as a payment method.
    '''

    entry_method = None
    '''
    Indicates how the card's track data was obtained.
    '''

    value = None
    '''
    The card's track data.
    '''



"""
Setup for GlobalPayments.Api
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='GlobalPayments.Api',
    version='1.0.4',
    author='Heartland Payment Systems',
    author_email='EntApp_DevPortal@e-hps.com',
    packages=[
        'globalpayments', 'globalpayments.api', 'globalpayments.api.builders',
        'globalpayments.api.builders.validations',
        'globalpayments.api.entities',
        'globalpayments.api.entities.table_service',
        'globalpayments.api.gateways', 'globalpayments.api.payment_methods',
        'globalpayments.api.services', 'globalpayments.api.utils'
    ],
    scripts=[],
    url='https://developer.heartlandpaymentsystems.com/',
    license='LICENSE.md',
    description='Global Payments Python SDK for integrating with Heartland and Global Payments merchant services APIs.',
    long_description=open('README.txt').read(),
    install_requires=[
        'xmltodict >= 0.9.0', 'jsonpickle >= 0.6.1', 'enum34 >= 1.1.6',
        'urllib3[secure] >= 1.18', 'certifi >= 2016.9.26',
        'pyopenssl >= 17.5.0'
    ])
